import 'package:dio/dio.dart';
import 'package:morph_dart/src/models/field/field.dart';
import 'package:morph_dart/src/models/table/table.dart';

class FieldMethods {
  final Dio dio;
  final String tableSlug;

  const FieldMethods(this.dio, this.tableSlug);

  Future<TableResponse> createField({
    required CreateFieldRequestBody body,
    int? linkedTableDepthMax,
  }) async {
    final response = await dio
        .post<Map<String, dynamic>>(
          '/field',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
          queryParameters: {
            'linkedTableDepthMax': linkedTableDepthMax,
          }..removeWhere((key, value) => value == null),
          data: body,
        )
        .then((res) => TableResponse.fromJson(res.data!));

    return response;
  }

  Future<TableResponse> updateField({
    required String fieldId,
    required UpdateFieldRequestBody body,
    int? linkedTableDepthMax,
  }) async {
    final response = await dio
        .put<Map<String, dynamic>>(
          '/field/$fieldId',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
          queryParameters: {
            'linkedTableDepthMax': linkedTableDepthMax,
          }..removeWhere((key, value) => value == null),
          data: body,
        )
        .then((res) => TableResponse.fromJson(res.data!));

    return response;
  }

  Future<bool> deleteField({
    required String fieldId,
  }) async {
    final response = await dio
        .delete<Map<String, dynamic>>(
          '/field/$fieldId',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
        )
        .then((res) => res.data!['message'] == 'ok');

    return response;
  }

  Future<TableResponse> createFieldOption({
    required String fieldId,
    required CreateFieldOptionRequestBody body,
    int? linkedTableDepthMax,
  }) async {
    final response = await dio
        .post<Map<String, dynamic>>(
          '/field/$fieldId/option',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
          queryParameters: {
            'linkedTableDepthMax': linkedTableDepthMax,
          }..removeWhere((key, value) => value == null),
          data: body,
        )
        .then((res) => TableResponse.fromJson(res.data!));

    return response;
  }

  Future<bool> deleteFieldOption({
    required String fieldId,
    required String optionId,
  }) async {
    final response = await dio
        .delete<Map<String, dynamic>>(
          '/field/$fieldId/option/$optionId',
          options: Options(
            headers: {
              'tableSlug': tableSlug,
            },
          ),
        )
        .then((res) => res.data!['message'] == 'ok');

    return response;
  }
}
