import 'package:dio/dio.dart';
import 'package:morph_dart/src/models/auth/auth.dart';

class GroupMethods {
  final Dio dio;

  const GroupMethods(this.dio);

  Future<UserpoolGroupListResponse> getGroups() async {
    final response = await dio
        .get<Map<String, dynamic>>('/group')
        .then((res) => UserpoolGroupListResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolGroupResponse> findGroup({
    required String groupId,
  }) async {
    final response = await dio
        .get<Map<String, dynamic>>('/group/$groupId')
        .then((res) => UserpoolGroupResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolGroupResponse> createGroup({
    required CreateUserpoolGroupRequestBody body,
  }) async {
    final response = await dio
        .post<Map<String, dynamic>>(
          '/group',
          data: body,
        )
        .then((res) => UserpoolGroupResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolGroupResponse> updateGroup({
    required String groupId,
    required UpdateUserpoolGroupRequestBody body,
  }) async {
    final response = await dio
        .put<Map<String, dynamic>>(
          '/group/$groupId',
          data: body,
        )
        .then((res) => UserpoolGroupResponse.fromJson(res.data!));

    return response;
  }

  Future<bool> deleteGroup({
    required String groupId,
  }) async {
    final response = await dio
        .delete<Map<String, dynamic>>('/group/$groupId')
        .then((res) => res.data!['message'] == 'ok');

    return response;
  }

  Future<UserpoolGroupResponse> createGroupUser({
    required String groupId,
    required String userpoolUserId,
    required CreateUserpoolGroupUserRequestBody body,
  }) async {
    final response = await dio
        .post<Map<String, dynamic>>(
          '/group/$groupId/user/$userpoolUserId',
          data: body,
        )
        .then((res) => UserpoolGroupResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolGroupResponse> updateGroupUser({
    required String groupId,
    required String userpoolUserId,
    required UpdateUserpoolGroupUserRequestBody body,
  }) async {
    final response = await dio
        .put<Map<String, dynamic>>(
          '/group/$groupId/user/$userpoolUserId',
          data: body,
        )
        .then((res) => UserpoolGroupResponse.fromJson(res.data!));

    return response;
  }

  Future<bool> deleteGroupUser({
    required String groupId,
    required String userpoolUserId,
  }) async {
    final response = await dio
        .delete<Map<String, dynamic>>('/group/$groupId/user/$userpoolUserId')
        .then((res) => res.data!['message'] == 'ok');

    return response;
  }
}
