import 'package:dio/dio.dart';
import 'package:morph_dart/src/models/auth/auth.dart';

class TeamMethods {
  final Dio dio;

  const TeamMethods(this.dio);

  Future<UserpoolUserResponse> createTeam({
    required CreateUserpoolTeamRequestBody body,
  }) async {
    final response = await dio
        .post<Map<String, dynamic>>(
          '/team',
          data: body,
        )
        .then((res) => UserpoolUserResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolUserTeamListResponse> getTeams({
    int? limit,
    int? skip,
    String? userpoolId,
  }) async {
    final response = await dio
        .get<Map<String, dynamic>>(
          '/team',
          queryParameters: {
            'limit': limit,
            'skip': skip,
            'userpoolId': userpoolId,
          }..removeWhere((key, value) => value == null),
        )
        .then((res) => UserpoolUserTeamListResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolTeamUserListResponse> inviteTeamUser({
    required InviteUserpoolTeamUserRequestBody body,
  }) async {
    final response = await dio
        .post<Map<String, dynamic>>(
          '/team/invite',
          data: body,
        )
        .then((res) => UserpoolTeamUserListResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolUserResponse> joinTeam({
    required JoinTeamRequestBody body,
  }) async {
    final response = await dio
        .post<Map<String, dynamic>>(
          '/team/join',
          data: body,
        )
        .then((res) => UserpoolUserResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolTeamUserListResponse> getTeamUsers({
    required String userpoolTeamId,
  }) async {
    final response = await dio
        .get<Map<String, dynamic>>('/team/$userpoolTeamId')
        .then((res) => UserpoolTeamUserListResponse.fromJson(res.data!));

    return response;
  }

  Future<UserpoolTeamUserListResponse> updateTeamUser({
    required UpdateUserpoolTeamUserRequestBody body,
  }) async {
    final response = await dio
        .put<Map<String, dynamic>>(
          '/team/user',
          data: body,
        )
        .then((res) => UserpoolTeamUserListResponse.fromJson(res.data!));

    return response;
  }
}
