import 'package:json_annotation/json_annotation.dart';
import 'package:morph_dart/src/enums/userpool.dart';

part 'auth.g.dart';

///==================================================================
/// Schema
///==================================================================
@JsonSerializable()
class UserpoolUserTeamObject {
  final String userpoolTeamId;
  final String teamName;
  final String? icon;
  final UserpoolUserInviteStatus? inviteStatus;
  final UserpoolUserType? userType;
  final bool? allowTableEdit;
  final int? status;

  const UserpoolUserTeamObject({
    required this.userpoolTeamId,
    required this.teamName,
    this.icon,
    this.inviteStatus,
    this.userType,
    this.allowTableEdit,
    this.status,
  });

  factory UserpoolUserTeamObject.fromJson(Map<String, Object?> json) =>
      _$UserpoolUserTeamObjectFromJson(json);
}

@JsonSerializable()
class _UserpoolUserBaseObject {
  final String userpoolUserId;
  final String email;
  final String? firstName;
  final String? lastName;
  final String? username;
  final String? icon;

  const _UserpoolUserBaseObject({
    required this.userpoolUserId,
    required this.email,
    this.firstName,
    this.lastName,
    this.username,
    this.icon,
  });
}

@JsonSerializable()
class UserpoolUserObject extends _UserpoolUserBaseObject {
  final List<UserpoolUserTeamObject> teams;

  const UserpoolUserObject({
    required super.userpoolUserId,
    required super.email,
    super.firstName,
    super.lastName,
    super.username,
    super.icon,
    required this.teams,
  });

  factory UserpoolUserObject.fromJson(Map<String, dynamic> json) =>
      _$UserpoolUserObjectFromJson(json);
}

@JsonSerializable()
class UserpoolTeamUserObject extends _UserpoolUserBaseObject {
  final UserpoolUserInviteStatus? inviteStatus;
  final UserpoolUserType userType;
  final bool allowTableEdit;
  final int status;

  const UserpoolTeamUserObject({
    required super.userpoolUserId,
    required super.email,
    super.firstName,
    super.lastName,
    super.username,
    super.icon,
    this.inviteStatus,
    required this.userType,
    required this.allowTableEdit,
    required this.status,
  });

  factory UserpoolTeamUserObject.fromJson(Map<String, dynamic> json) =>
      _$UserpoolTeamUserObjectFromJson(json);
}

@JsonSerializable()
class UserpoolGroupUserObject extends _UserpoolUserBaseObject {
  final bool isGroupAdmin;

  const UserpoolGroupUserObject({
    required super.userpoolUserId,
    required super.email,
    super.firstName,
    super.lastName,
    super.username,
    super.icon,
    required this.isGroupAdmin,
  });

  factory UserpoolGroupUserObject.fromJson(Map<String, dynamic> json) =>
      _$UserpoolGroupUserObjectFromJson(json);
}

@JsonSerializable()
class UserpoolGroupObject {
  final String groupId;
  final String groupName;
  final String? icon;
  final int status;
  final List<UserpoolGroupUserObject> groupUsers;
  final String createdAt;
  final String? updatedAt;

  const UserpoolGroupObject({
    required this.groupId,
    required this.groupName,
    this.icon,
    required this.status,
    required this.groupUsers,
    required this.createdAt,
    this.updatedAt,
  });

  factory UserpoolGroupObject.fromJson(Map<String, dynamic> json) =>
      _$UserpoolGroupObjectFromJson(json);
}

///==================================================================
/// Request Body
///==================================================================
@JsonSerializable()
class SignUpRequestBody {
  final String firstName;
  final String lastName;
  final String username;
  final String? icon;

  const SignUpRequestBody({
    required this.firstName,
    required this.lastName,
    required this.username,
    this.icon,
  });

  Map<String, dynamic> toJson() => _$SignUpRequestBodyToJson(this);
}

@JsonSerializable()
class UpdateMeRequestBody {
  final String firstName;
  final String lastName;
  final String username;
  final String? icon;

  const UpdateMeRequestBody({
    required this.firstName,
    required this.lastName,
    required this.username,
    this.icon,
  });

  Map<String, dynamic> toJson() => _$UpdateMeRequestBodyToJson(this);
}

@JsonSerializable()
class CreateUserpoolTeamRequestBody {
  final String teamName;
  final String? icon;

  const CreateUserpoolTeamRequestBody({
    required this.teamName,
    this.icon,
  });

  Map<String, dynamic> toJson() => _$CreateUserpoolTeamRequestBodyToJson(this);
}

@JsonSerializable()
class InviteUserpoolTeamUserRequestBody {
  final String userpoolTeamId;
  final String email;

  const InviteUserpoolTeamUserRequestBody({
    required this.userpoolTeamId,
    required this.email,
  });

  Map<String, dynamic> toJson() =>
      _$InviteUserpoolTeamUserRequestBodyToJson(this);
}

@JsonSerializable()
class JoinTeamRequestBody {
  final String userpoolTeamId;

  const JoinTeamRequestBody({
    required this.userpoolTeamId,
  });

  Map<String, dynamic> toJson() => _$JoinTeamRequestBodyToJson(this);
}

@JsonSerializable()
class UpdateUserpoolTeamUserRequestBody {
  final String userId;
  final String userpoolTeamId;
  final bool allowEditTable;
  final UserpoolUserType userType;

  const UpdateUserpoolTeamUserRequestBody({
    required this.userId,
    required this.userpoolTeamId,
    required this.allowEditTable,
    required this.userType,
  });

  Map<String, dynamic> toJson() =>
      _$UpdateUserpoolTeamUserRequestBodyToJson(this);
}

@JsonSerializable()
class CreateUserpoolGroupRequestBody {
  final String groupName;
  final int status;
  final UserpoolGroupTarget targetType;
  final String? icon;

  const CreateUserpoolGroupRequestBody({
    required this.groupName,
    required this.status,
    required this.targetType,
    this.icon,
  });

  Map<String, dynamic> toJson() => _$CreateUserpoolGroupRequestBodyToJson(this);
}

@JsonSerializable()
class UpdateUserpoolGroupRequestBody {
  final String groupName;
  final int status;
  final String? icon;

  const UpdateUserpoolGroupRequestBody({
    required this.groupName,
    required this.status,
    this.icon,
  });

  Map<String, dynamic> toJson() => _$UpdateUserpoolGroupRequestBodyToJson(this);
}

@JsonSerializable()
class CreateUserpoolGroupUserRequestBody {
  final bool isAdmin;

  const CreateUserpoolGroupUserRequestBody({
    required this.isAdmin,
  });

  Map<String, dynamic> toJson() =>
      _$CreateUserpoolGroupUserRequestBodyToJson(this);
}

@JsonSerializable()
class UpdateUserpoolGroupUserRequestBody {
  final bool isAdmin;

  const UpdateUserpoolGroupUserRequestBody({
    required this.isAdmin,
  });

  Map<String, dynamic> toJson() =>
      _$UpdateUserpoolGroupUserRequestBodyToJson(this);
}

///==================================================================
/// Response
///==================================================================
@JsonSerializable()
class UserpoolUserResponse extends UserpoolUserObject {
  const UserpoolUserResponse({
    required super.userpoolUserId,
    required super.email,
    super.firstName,
    super.lastName,
    super.username,
    super.icon,
    required super.teams,
  });

  factory UserpoolUserResponse.fromJson(Map<String, dynamic> json) =>
      _$UserpoolUserResponseFromJson(json);
}

@JsonSerializable()
class UserpoolTeamUserResponse extends UserpoolTeamUserObject {
  const UserpoolTeamUserResponse({
    required super.userpoolUserId,
    required super.email,
    super.firstName,
    super.lastName,
    super.username,
    super.icon,
    super.inviteStatus,
    required super.userType,
    required super.allowTableEdit,
    required super.status,
  });

  factory UserpoolTeamUserResponse.fromJson(Map<String, dynamic> json) =>
      _$UserpoolTeamUserResponseFromJson(json);
}

@JsonSerializable()
class UserpoolTeamUserListResponse {
  final List<UserpoolTeamUserObject> users;

  const UserpoolTeamUserListResponse({
    required this.users,
  });

  factory UserpoolTeamUserListResponse.fromJson(Map<String, dynamic> json) =>
      _$UserpoolTeamUserListResponseFromJson(json);
}

@JsonSerializable()
class UserpoolUserTeamListResponse {
  final List<UserpoolUserTeamObject> teams;
  final int count;

  const UserpoolUserTeamListResponse({
    required this.teams,
    required this.count,
  });

  factory UserpoolUserTeamListResponse.fromJson(Map<String, dynamic> json) =>
      _$UserpoolUserTeamListResponseFromJson(json);
}

@JsonSerializable()
class UserpoolGroupResponse extends UserpoolGroupObject {
  const UserpoolGroupResponse({
    required super.groupId,
    required super.groupName,
    super.icon,
    required super.status,
    required super.groupUsers,
    required super.createdAt,
    super.updatedAt,
  });

  factory UserpoolGroupResponse.fromJson(Map<String, dynamic> json) =>
      _$UserpoolGroupResponseFromJson(json);
}

@JsonSerializable()
class UserpoolGroupListResponse {
  final List<UserpoolGroupObject> data;
  final int count;

  UserpoolGroupListResponse({
    required this.data,
    required this.count,
  });

  factory UserpoolGroupListResponse.fromJson(Map<String, dynamic> json) =>
      _$UserpoolGroupListResponseFromJson(json);
}
