import 'package:json_annotation/json_annotation.dart';
import 'package:morph_dart/src/enums/record.dart';

part 'filter_condition.g.dart';

abstract class _FilterConditionBase {
  const _FilterConditionBase();

  Map<String, dynamic> toJson();
}

abstract class FilterConditionUnit extends _FilterConditionBase {
  const FilterConditionUnit() : super();

  const factory FilterConditionUnit.text({
    required String property,
    required TextFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitText;

  const factory FilterConditionUnit.select({
    required String property,
    required SelectFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitSelect;

  const factory FilterConditionUnit.number({
    required String property,
    required NumberFilterOperator operator,
    num? value,
  }) = _FilterConditionUnitNumber;

  const factory FilterConditionUnit.boolean({
    required String property,
    required BooleanFilterOperator operator,
    bool? value,
  }) = _FilterConditionUnitBoolean;

  const factory FilterConditionUnit.date({
    required String property,
    required DateFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitDate;

  const factory FilterConditionUnit.url({
    required String property,
    required UrlFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitUrl;

  const factory FilterConditionUnit.email({
    required String property,
    required EmailFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitEmail;

  const factory FilterConditionUnit.table({
    required String property,
    required TableFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitTable;

  const factory FilterConditionUnit.phoneNumber({
    required String property,
    required PhoneNumberFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitPhoneNumber;

  const factory FilterConditionUnit.userpool({
    required String property,
    required UserpoolFilterOperator operator,
    String? value,
  }) = _FilterConditionUnitUserpool;
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitText extends FilterConditionUnit {
  final String property;
  final TextFilterOperator operator;
  final String? value;

  const _FilterConditionUnitText({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitTextToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitSelect extends FilterConditionUnit {
  final String property;
  final SelectFilterOperator operator;
  final String? value;

  const _FilterConditionUnitSelect({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitSelectToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitNumber extends FilterConditionUnit {
  final String property;
  final NumberFilterOperator operator;
  final num? value;

  const _FilterConditionUnitNumber({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitNumberToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitBoolean extends FilterConditionUnit {
  final String property;
  final BooleanFilterOperator operator;
  final bool? value;

  const _FilterConditionUnitBoolean({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitBooleanToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitDate extends FilterConditionUnit {
  final String property;
  final DateFilterOperator operator;
  final String? value;

  const _FilterConditionUnitDate({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitDateToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitUrl extends FilterConditionUnit {
  final String property;
  final UrlFilterOperator operator;
  final String? value;

  const _FilterConditionUnitUrl({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitUrlToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitEmail extends FilterConditionUnit {
  final String property;
  final EmailFilterOperator operator;
  final String? value;

  const _FilterConditionUnitEmail({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitEmailToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitTable extends FilterConditionUnit {
  final String property;
  final TableFilterOperator operator;
  final String? value;

  const _FilterConditionUnitTable({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitTableToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitPhoneNumber extends FilterConditionUnit {
  final String property;
  final PhoneNumberFilterOperator operator;
  final String? value;

  const _FilterConditionUnitPhoneNumber({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitPhoneNumberToJson(this);
}

@JsonSerializable(createFactory: false)
class _FilterConditionUnitUserpool extends FilterConditionUnit {
  final String property;
  final UserpoolFilterOperator operator;
  final String? value;

  const _FilterConditionUnitUserpool({
    required this.property,
    required this.operator,
    this.value,
  }) : super();

  @override
  Map<String, dynamic> toJson() => _$FilterConditionUnitUserpoolToJson(this);
}

/// FilterCondition
abstract class FilterCondition extends _FilterConditionBase {
  const FilterCondition();

  static const converter = FilterConditionJsonConverter();

  @override
  Map<String, dynamic> toJson() => converter.toJson(this)!;

  const factory FilterCondition.and(
    List<_FilterConditionBase> and,
  ) = _FilterConditionAnd;

  const factory FilterCondition.or(
    List<_FilterConditionBase> or,
  ) = _FilterConditionOr;
}

class _FilterConditionAnd extends FilterCondition {
  final List<_FilterConditionBase> and;

  const _FilterConditionAnd(this.and) : super();
}

class _FilterConditionOr extends FilterCondition {
  final List<_FilterConditionBase> or;

  const _FilterConditionOr(this.or) : super();
}

///==================================================================
/// JSON Converter
///==================================================================
class FilterConditionJsonConverter
    implements JsonConverter<FilterCondition?, Map<String, dynamic>?> {
  const FilterConditionJsonConverter();

  @override
  FilterCondition? fromJson(Map<String, dynamic>? json) {
    throw Exception('Do not use FilterCondition.fromJson().');
  }

  @override
  Map<String, dynamic>? toJson(FilterCondition? data) {
    if (data == null) {
      return null;
    } else if (data is _FilterConditionAnd) {
      return {
        'and': data.and.map((item) => item.toJson()).toList(),
      };
    } else if (data is _FilterConditionOr) {
      return {
        'or': data.or.map((item) => item.toJson()).toList(),
      };
    } else {
      throw Exception(
        "This method cannot be used for types that are neither 'FilterConditionAnd' nor 'FilterConditionOr'.",
      );
    }
  }

  static Map<String, dynamic>? staticToJson(FilterCondition? data) =>
      const FilterConditionJsonConverter().toJson(data);
}
