import 'package:json_annotation/json_annotation.dart';
import 'package:morph_dart/morph_dart.dart';

part 'record.g.dart';

///==================================================================
/// Schema
///==================================================================
@JsonSerializable()
class SortCondition {
  final String property;
  final SortConditionDirection direction;

  const SortCondition({
    required this.property,
    required this.direction,
  });

  Map<String, dynamic> toJson() => _$SortConditionToJson(this);

  factory SortCondition.fromJson(Map<String, dynamic> json) =>
      _$SortConditionFromJson(json);
}

@JsonSerializable(genericArgumentFactories: true)
class RecordObject<Fields> {
  final String id;
  final Fields fields;
  final UserpoolTeamUserObject? createdBy;
  final String createdAt;
  final String? updateAt;

  const RecordObject({
    required this.id,
    required this.fields,
    this.createdBy,
    required this.createdAt,
    this.updateAt,
  });

  factory RecordObject.fromJson(
    Map<String, dynamic> json,
    Fields Function(Object?) fromJsonFields,
  ) =>
      _$RecordObjectFromJson(json, fromJsonFields);
}

///==================================================================
/// Request Body
///==================================================================
@JsonSerializable()
class QueryRecordRequestBody {
  final List<String>? select;
  @FilterCondition.converter
  final FilterCondition? filter;
  final List<SortCondition>? sorts;
  final int? limit;
  final int? skip;
  final String? userpoolTeamId;
  final String? groupId;
  final bool? withNextLayer;

  const QueryRecordRequestBody({
    this.select,
    this.filter,
    this.sorts,
    this.limit,
    this.skip,
    this.userpoolTeamId,
    this.groupId,
    this.withNextLayer,
  });

  Map<String, dynamic> toJson() => _$QueryRecordRequestBodyToJson(this);
}

@JsonSerializable(explicitToJson: true)
class DownloadRecordRequestBody extends QueryRecordRequestBody {
  final DownloadFormat? format;

  const DownloadRecordRequestBody({
    List<String>? select,
    FilterCondition? filter,
    List<SortCondition>? sorts,
    int? limit,
    int? skip,
    String? userpoolTeamId,
    String? groupId,
    bool? withNextLayer,
    this.format,
  }) : super(
          select: select,
          filter: filter,
          sorts: sorts,
          limit: limit,
          skip: skip,
          userpoolTeamId: userpoolTeamId,
          groupId: groupId,
          withNextLayer: withNextLayer,
        );

  @override
  Map<String, dynamic> toJson() => _$DownloadRecordRequestBodyToJson(this);
}

@JsonSerializable(genericArgumentFactories: true)
class CreateRecordRequestBody<Fields> {
  final Fields data;
  final String? userpoolTeamId;
  final String? groupId;

  const CreateRecordRequestBody({
    required this.data,
    this.userpoolTeamId,
    this.groupId,
  });

  Map<String, dynamic> toJson(Object Function(Fields value) toJsonFields) =>
      _$CreateRecordRequestBodyToJson(this, toJsonFields);
}

@JsonSerializable(genericArgumentFactories: true)
class UpdateRecordRequestBody<Fields> {
  final Fields data;

  const UpdateRecordRequestBody({
    required this.data,
  });

  Map<String, dynamic> toJson(Object Function(Fields value) toJsonFields) =>
      _$UpdateRecordRequestBodyToJson(this, toJsonFields);
}

///==================================================================
/// Response
///==================================================================
@JsonSerializable(genericArgumentFactories: true)
class RecordResponse<Fields> extends RecordObject<Fields> {
  const RecordResponse({
    required super.id,
    required super.fields,
    super.createdBy,
    required super.createdAt,
    super.updateAt,
  });

  factory RecordResponse.fromJson(
    Map<String, dynamic> json,
    Fields Function(Object?) fromJsonFields,
  ) =>
      _$RecordResponseFromJson(json, fromJsonFields);
}

@JsonSerializable(genericArgumentFactories: true)
class RecordListResponse<Fields> {
  final List<RecordObject<Fields>> records;
  final int count;

  const RecordListResponse({required this.records, required this.count});

  factory RecordListResponse.fromJson(
    Map<String, dynamic> json,
    Fields Function(Object?) fromJsonFields,
  ) =>
      _$RecordListResponseFromJson(json, fromJsonFields);
}

@JsonSerializable()
class DownloadRecordResponse {
  final String url;
  final DownloadFormat format;
  final int count;

  const DownloadRecordResponse({
    required this.url,
    required this.format,
    required this.count,
  });

  factory DownloadRecordResponse.fromJson(Map<String, dynamic> json) =>
      _$DownloadRecordResponseFromJson(json);
}
