import 'package:dio/dio.dart';
import 'package:morph_dart/src/error/network_error.dart';

class ErrorInterceptor extends QueuedInterceptor {
  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    return handler.next(generateMorphNetworkError(err));
  }

  @override
  Future<void> onResponse(
    Response<dynamic> response,
    ResponseInterceptorHandler handler,
  ) async {
    if (response.data is Map<String, dynamic> &&
        (response.data as Map<String, dynamic>)['error'] != null) {
      // Error Response
      return handler.reject(
        generateMorphNetworkResponseError(
          response.requestOptions,
          response.data as Map<String, dynamic>,
        ),
      );
    }
    return handler.next(response);
  }
}
