// ignore_for_file: avoid_print

import 'package:dotenv/dotenv.dart';
import 'package:morph_dart/morph_dart.dart';
import 'package:test/test.dart';

import 'fields/test_fields.dart';

void main() async {
  final env = DotEnv()..load();

  test('env', () {
    print('MORPH_APPLICATION_ID: ${env['MORPH_APPLICATION_ID']}');
    print('MORPH_BASE_URL: ${env['MORPH_BASE_URL']}');
    print('MORPH_API_KEY: ${env['MORPH_API_KEY']}');
    expect(
      env.isEveryDefined(
        ['MORPH_APPLICATION_ID', 'MORPH_BASE_URL', 'MORPH_API_KEY'],
      ),
      true,
    );
  });

// TODO: test case for acl='api'
  final morphClient = MorphClient(
    MorphClientOptions(
      applicationId: env['MORPH_APPLICATION_ID'] ?? '',
      baseUrl: env['MORPH_BASE_URL'] ?? '',
      apiKey: env['MORPH_API_KEY'] ?? '',
      tokenConfig: AuthTokenConfig(
        getIdToken: () => '',
        refresh: () {},
      ),
    ),
  );

  // final testTableSlug =
  //     'test-table-slug-${DateTime.now().millisecondsSinceEpoch} ';
  // final testTableName = testTableSlug;

  // group('table methods', () {
  //   test('create table', () async {
  //     final createTableResponse = await morphClient.methods.table().createTable(
  //           body: CreateTableRequestBody(
  //             tableSlug: testTableSlug,
  //             tableName: testTableName,
  //             tableType: TableType.general,
  //             isEditable: false,
  //             readPermission: TablePermission.team,
  //             writePermission: TablePermission.team,
  //           ),
  //         );
  //     expect(createTableResponse.tableSlug, testTableSlug);
  //     expect(createTableResponse.tableName, testTableName);
  //   });

  //   test('find table', () async {
  //     final findTableResponse =
  //         await morphClient.methods.table().findTable(tableSlug: testTableSlug);
  //     expect(findTableResponse.tableSlug, testTableSlug);
  //     expect(findTableResponse.tableName, testTableName);
  //   });

  //   test('get tables', () async {
  //     final tableListResponse =
  //         await morphClient.methods.table().getTables(limit: 5);
  //     expect(tableListResponse.count.runtimeType, int);
  //   });
  // });

  group('record methods', () {
    final testTableRecordMethods = morphClient.methods.record<TestFields>(
      'test-table',
      TestFields.toJson,
      // ignore: cast_nullable_to_non_nullable
      (json) => TestFields.fromJson(json),
    );

    test('query record', () async {
      final queryRecordResponse = await testTableRecordMethods.queryRecords(
        body: const QueryRecordRequestBody(),
      );
      expect(queryRecordResponse.count.runtimeType, int);
    });

    late final String createdRecordId;

    test('create record', () async {
      final createRecordResponse = await testTableRecordMethods.createRecord(
        body: const CreateRecordRequestBody(
          data: TestFields(fieldBoolean: true),
        ),
      );
      expect(createRecordResponse.id.runtimeType, String);
      createdRecordId = createRecordResponse.id;
    });

    test('find record', () async {
      final findRecordResponse = await testTableRecordMethods.findRecord(
        id: createdRecordId,
      );
      expect(findRecordResponse.runtimeType, isNotNull);
      expect(findRecordResponse!.id, createdRecordId);
    });

    test('update record', () async {
      const updateValue = false;
      final updateRecordResponse = await testTableRecordMethods.updateRecord(
        recordId: createdRecordId,
        body: const UpdateRecordRequestBody<TestFields>(
          data: TestFields(fieldBoolean: updateValue),
        ),
      );
      expect(updateRecordResponse.fields.fieldBoolean, updateValue);
    });
    test('delete record', () async {
      final deleteRecordResponse = await testTableRecordMethods.deleteRecord(
        recordId: createdRecordId,
      );
      expect(deleteRecordResponse, true);
    });
  });
}
